@@ -2,10 +2,10 @@ from ploomber.dag.DAG import DAG
 from ploomber.dag.DAGConfigurator import DAGConfigurator
 from ploomber.env.env import Env
 from ploomber.env.decorators import load_env, with_env
-from ploomber.templates.SourceLoader import SourceLoader
+from ploomber.placeholders.SourceLoader import SourceLoader
 
 
-__version__ = '0.3.5'
+__version__ = '0.4.0'
 
 # Set default logging handler to avoid "No handler found" warnings.
 import logging
