@@ -1,9 +1,33 @@
 """
-DAG module
+Ploomber execution model:
+
+To orchestrate Task execution, they are organized in a DAG object that keeps
+track of dependencies and status.
+
+It all starts with a call to DAG.render(), all placeholders are resolved
+and the task status is determined. If any task fails to render, the process
+is stopped and such DAG cannot be executed.
+
+Once all tasks render successfully, their product's metadata determined
+whether they should run or not based on source code changes, this behavior
+can be overridden by passing DAG.render(force=True).
+
+When DAG.build() is called, actual execution happens. Tasks are executed in
+order (but up-to-date tasks are skipped). If tasks succeed, their downstream
+dependencies will be executed, otherwise they are aborted.
+
+If any tasks fail, DAG executes the on_failure hook, otherwise, it executes
+the on_finish hook.
+
+Tasks support hooks as well and building a task is a process by itself, see
+the docstring in the Task module for details.
+
+The DAG does not execute tasks, but delegates this to an Executor object,
+executors adhere to task status and do not build tasks if they are marked
+as Aborted or Skipped.
 
-A DAG is collection of tasks that makes sure they are executed in
-the right order
 """
+from collections.abc import Iterable
 import traceback
 from copy import copy, deepcopy
 from pathlib import Path
@@ -11,6 +35,7 @@ import warnings
 import logging
 import collections
 import tempfile
+from math import ceil
 
 try:
     import importlib.resources as importlib_resources
@@ -31,7 +56,8 @@ from ploomber.util import (image_bytes2html, isiterable, path2fig, requires,
 from ploomber import resources
 from ploomber import executors
 from ploomber.constants import TaskStatus, DAGStatus
-from ploomber.exceptions import DAGBuildError, DAGRenderError
+from ploomber.exceptions import (DAGBuildError, DAGRenderError,
+                                 DAGBuildEarlyStop)
 from ploomber.MessageCollector import MessageCollector
 from ploomber.util.util import callback_check
 from ploomber.dag.DAGConfiguration import DAGConfiguration
@@ -56,6 +82,28 @@ class DAG(collections.abc.Mapping):
         ploomber.executors.Parallel), is a string is passed ('serial'
         or 'parallel') the corresponding executor is initialized with default
         parameters
+
+    Attributes
+    ----------
+    name : str
+        A name to identify the DAG
+
+    clients : dict
+        A class to client mapping
+
+    executor : ploomber.Executor
+        Executor object to run tasks
+
+    on_finish : callable
+        Function to execute upon execution. Can request a "dag" parameter
+        and/or "report", which containes the report object returned by the
+        build function.
+
+    on_failure : callable
+        Function to execute upon failure. Can request a "dag" parameter and/or
+        "traceback" which will contain a dictionary, possible keys are "build"
+        which contains the build error traceback and "on_finish" which contains
+        the on_finish hook traceback, if any.
     """
 
     def __init__(self, name=None, clients=None, executor='serial'):
@@ -89,6 +137,14 @@ class DAG(collections.abc.Mapping):
         # task access differ using .dag.differ
         self.differ = self._params.differ
 
+    @property
+    def executor(self):
+        return self._executor
+
+    @executor.setter
+    def executor(self, value):
+        self._executor = value
+
     @property
     def _exec_status(self):
         return self.__exec_status
@@ -164,6 +220,8 @@ class DAG(collections.abc.Mapping):
 
     @property
     def product(self):
+        # NOTE: this allows declaring a dag as a dependency for one task,
+        # maybe create a metaclass that applies to DAGs and Task
         # We have to rebuild it since tasks might have been added
         return MetaProduct([t.product for t in self.values()])
 
@@ -179,7 +237,11 @@ class DAG(collections.abc.Mapping):
         return t
 
     def render(self, force=False, show_progress=True):
-        """Render the graph
+        """
+
+        Render resolves all placeholders in tasks and determines whether
+        a task should run or not based on the task.product metadata, this
+        allows up-to-date tasks to be skipped.
         """
         g = self._to_graph()
 
@@ -226,10 +288,15 @@ class DAG(collections.abc.Mapping):
             A dict-like object with tasks as keys and dicts with task
             status as values
         """
-        kwargs = callback_check(self._params.logging_handler_factory,
+        kwargs = callback_check(self._params.logging_factory,
                                 available={'dag_name': self.name})
 
-        dag_logger = DAGLogger(self._params.logging_handler_factory(**kwargs))
+        res = self._params.logging_factory(**kwargs)
+
+        if isinstance(res, Iterable):
+            dag_logger = DAGLogger(*res)
+        else:
+            dag_logger = DAGLogger(handler=res)
 
         with dag_logger:
             report = self._build(force, show_progress)
@@ -253,55 +320,120 @@ class DAG(collections.abc.Mapping):
 
             self._logger.info('Building DAG %s', self)
 
+            tb = {}
+
             try:
                 # within_dag flags when we execute a task in isolation
                 # vs as part of a dag execution
+                # FIXME: not passing force flag
                 task_reports = self._executor(dag=self,
-                                              show_progress=show_progress,
-                                              task_kwargs=dict(within_dag=True))
-            except Exception as e:
+                                              show_progress=show_progress)
+
+            # executors raise this error to signal that there was an error
+            # building the dag, this allows us to run the on_failure hook,
+            # but any other errors should not be caught (e.g.
+            # a user might turn that setting off in the executor to start
+            # a debugging session at the line of failure)
+            except DAGBuildError as e:
+                tb['build'] = traceback.format_exc()
+                # error build dag, log exception and set status
+                self._logger.exception('Failure when building DAG "{}"'
+                                       .format(self.name))
                 self._exec_status = DAGStatus.Errored
-                e_new = DAGBuildError('Failed to build DAG {}'.format(self))
-
-                if self.on_failure:
-                    self._logger.debug('Executing on_failure hook '
-                                       'for dag "%s"', self.name)
-                    kwargs_available = copy(self._available_callback_kwargs)
-                    kwargs_available['traceback'] = traceback.format_exc()
-
-                    kwargs = callback_check(self.on_failure, kwargs_available)
-                    self.on_failure(**kwargs)
-                else:
-                    self._logger.debug('No on_failure hook for dag '
-                                       '"%s", skipping', self.name)
-
-                raise e_new from e
+                build_exception = e
+            except DAGBuildEarlyStop:
+                # early stop and empty on_failure, nothing left to do
+                if self.on_failure is None:
+                    # FIXME: remove this, only needed after rendering
+                    self._clear_cached_status()
+                    return
             else:
-                self._exec_status = DAGStatus.Executed
-            finally:
-                # always clear out status
+                # no error when building dag
+                build_exception = None
+
+            if build_exception is None:
+                empty = [TaskReport.empty_with_name(t.name)
+                         for t in self.values()
+                         if t.exec_status == TaskStatus.Skipped]
+
+                build_report = BuildReport(task_reports + empty)
+                self._logger.info(' DAG report:\n{}'.format(build_report))
+
+                # try on_finish hook
+                try:
+                    self._run_on_finish(build_report)
+                except Exception as e:
+                    tb['on_finish'] = traceback.format_exc()
+                    # on_finish error, log exception and set status
+                    msg = ('Exception when running on_finish '
+                           'for DAG "{}": {}'
+                           .format(self.name, e))
+                    self._logger.exception(msg)
+                    self._exec_status = DAGStatus.Errored
+
+                    if isinstance(e, DAGBuildEarlyStop):
+                        # early stop, nothing left to co
+                        # FIXME: remove this, only needed after rendering
+                        self._clear_cached_status()
+                        return
+                    else:
+                        # otherwise raise exception
+                        raise DAGBuildError(msg) from e
+                else:
+                    # DAG success and on_finish did not raise exception
+                    self._exec_status = DAGStatus.Executed
+                    # FIXME: remove this, only needed after rendering
+                    self._clear_cached_status()
+                    return build_report
+
+            else:
+                # DAG raised error, run on_failure hook
+                try:
+                    self._run_on_failure(tb)
+                except Exception as e:
+                    # error in hook, log exception
+                    msg = ('Exception when running on_failure '
+                           'for DAG "{}": {}'
+                           .format(self.name, e))
+                    self._logger.exception(msg)
+
+                    # do not raise exception if early stop
+                    if isinstance(e, DAGBuildEarlyStop):
+                        return
+                    else:
+                        raise DAGBuildError(msg) from e
+
+                # on_failure hook executed, raise original exception
+                # FIXME: remove this, only needed after rendering
                 self._clear_cached_status()
 
-            # add reports from skipped tasks
-            empty = [TaskReport.empty_with_name(t.name)
-                     for t in self.values()
-                     if t.exec_status == TaskStatus.Skipped]
+                raise DAGBuildError(
+                    'Failed to build DAG {}'.format(self)) from build_exception
 
-            build_report = BuildReport(task_reports + empty)
-            self._logger.info(' DAG report:\n{}'.format(build_report))
+    def _run_on_failure(self, tb):
+        if self.on_failure:
+            self._logger.debug('Executing on_failure hook '
+                               'for dag "%s"', self.name)
+            kwargs_available = copy(self._available_callback_kwargs)
+            kwargs_available['traceback'] = tb
 
-            if self.on_finish:
-                self._logger.debug('Executing on_finish hook '
-                                   'for dag "%s"', self.name)
-                kwargs_available = copy(self._available_callback_kwargs)
-                kwargs_available['report'] = build_report
-                kwargs = callback_check(self.on_finish, kwargs_available)
-                self.on_finish(**kwargs)
-            else:
-                self._logger.debug('No on_finish hook for dag '
-                                   '"%s", skipping', self.name)
+            kwargs = callback_check(self.on_failure, kwargs_available)
+            self.on_failure(**kwargs)
+        else:
+            self._logger.debug('No on_failure hook for dag '
+                               '"%s", skipping', self.name)
 
-            return build_report
+    def _run_on_finish(self, build_report):
+        if self.on_finish:
+            self._logger.debug('Executing on_finish hook '
+                               'for dag "%s"', self.name)
+            kwargs_available = copy(self._available_callback_kwargs)
+            kwargs_available['report'] = build_report
+            kwargs = callback_check(self.on_finish, kwargs_available)
+            self.on_finish(**kwargs)
+        else:
+            self._logger.debug('No on_finish hook for dag '
+                               '"%s", skipping', self.name)
 
     def build_partially(self, target, force=False, show_progress=True):
         """Partially build a dag until certain task
@@ -396,9 +528,9 @@ class DAG(collections.abc.Mapping):
 
         G = self._to_graph()
 
-        for n, data in G.nodes(data=True):
-            data['color'] = 'red' if n.product._is_outdated() else 'green'
-            data['label'] = n._short_repr()
+        for task, data in G.nodes(data=True):
+            data['color'] = 'red' if task.product._is_outdated() else 'green'
+            data['label'] = _task_short_repr(task)
 
         # https://networkx.github.io/documentation/networkx-1.10/reference/drawing.html
         # # http://graphviz.org/doc/info/attrs.html
@@ -406,6 +538,10 @@ class DAG(collections.abc.Mapping):
         G_ = nx.nx_agraph.to_agraph(G)
         G_.draw(path, prog='dot', args='-Grankdir=LR')
 
+        # plot function uses _is_outdated, which casues caching, clear up
+        # to avoid re-using this in any other operation
+        self._clear_cached_status()
+
         if output == 'matplotlib':
             return path2fig(path)
         else:
@@ -424,6 +560,10 @@ class DAG(collections.abc.Mapping):
             exceptions = MessageCollector()
             warnings_ = MessageCollector()
 
+            # reset all tasks status
+            for task in self.values():
+                task.exec_status = TaskStatus.WaitingRender
+
             for t in tasks:
                 # no need to process task with AbortedRender
                 if t.exec_status == TaskStatus.AbortedRender:
@@ -529,6 +669,7 @@ class DAG(collections.abc.Mapping):
         return {u: self._G.nodes[u]['task'] for u in upstream}
 
     def _clear_cached_status(self):
+        # NOTE: maybe make this a context manager and/or a decorator
         self._logger.debug('Clearing product status')
         # clearing out this way is only useful after building, but not
         # if the metadata changed since it wont be reloaded
@@ -538,6 +679,12 @@ class DAG(collections.abc.Mapping):
     def __getitem__(self, key):
         return self._G.nodes[key]['task']
 
+    def __delitem__(self, key):
+        # TODO: this implementation is correct but perhaps we should raise
+        # warning if the deleted task has downstream dependencies, render
+        # and build will no longer work
+        return self._G.remove_node(key)
+
     def __iter__(self):
         """Iterate task names in topological order
         """
@@ -574,3 +721,41 @@ class DAG(collections.abc.Mapping):
         self.__dict__.update(state)
         self._logger = logging.getLogger('{}.{}'.format(__name__,
                                                         type(self).__name__))
+
+
+def _single_product_short_repr(product):
+    s = str(product._identifier)
+
+    if len(s) > 20:
+        s_short = ''
+
+        t = ceil(len(s) / 20)
+
+        for i in range(t):
+            s_short += s[(20 * i):(20 * (i + 1))] + '\n'
+    else:
+        s_short = s
+
+    return s_short
+
+
+def _meta_product_short_repr(metaproduct):
+    return ', '.join([_single_product_short_repr(p)
+                      for p in metaproduct.products])
+
+
+def _product_short_repr(product):
+    if isinstance(product, MetaProduct):
+        return _meta_product_short_repr(product)
+    else:
+        return _single_product_short_repr(product)
+
+
+def _task_short_repr(task):
+    def short(s):
+        max_l = 30
+        return s if len(s) <= max_l else s[:max_l - 3] + '...'
+
+    return ('{} -> \n{}'
+            .format(short(str(task.name)),
+                    _product_short_repr(task.product)))
