@@ -1,7 +1,8 @@
 from ploomber.products.Product import Product
 from ploomber.products.MetaProduct import MetaProduct
 from ploomber.products.File import File
+from ploomber.products.GenericProduct import GenericProduct, GenericSQLRelation
 from ploomber.products.sql import SQLiteRelation, PostgresRelation
 
 __all__ = ['File', 'MetaProduct', 'Product', 'SQLiteRelation',
-           'PostgresRelation']
+           'PostgresRelation', 'GenericProduct', 'GenericSQLRelation']
