@@ -5,12 +5,39 @@ from jinja2 import Template
 
 from ploomber.products import Product
 from ploomber.products.serializers import Base64Serializer
-from ploomber.templates.Placeholder import SQLRelationPlaceholder
+from ploomber.placeholders.Placeholder import SQLRelationPlaceholder
 
 
-class SQLiteBackedProductMixin:
+class ProductWithClientMixin:
+    """
+    Adds the client property to a Product with the hierarchical resolution
+    logic: Product -> Task -> DAG.clients. Product.client is only used
+    for storing metadata
+    """
+    @property
+    def client(self):
+        # FIXME: this nested reference looks ugly, how can we improve this?
+        if self._client is None:
+            if self._task is None:
+                raise ValueError('Cannot obtain client for this product, '
+                                 'the constructor did not receive a client '
+                                 'and this product has not been assigned '
+                                 'to a DAG yet, hence we cannot look up '
+                                 'dag.clients')
+
+            default = self.task.dag.clients.get(type(self))
+
+            if default is None:
+                raise ValueError('{} must be initialized with a client'
+                                 .format(type(self).__name__))
+            else:
+                self._client = default
+
+        return self._client
+
+
+class SQLiteBackedProductMixin(ProductWithClientMixin):
     def _create_metadata_relation(self):
-
         create_metadata = """
         CREATE TABLE IF NOT EXISTS _metadata (
             name TEXT NOT NULL,
@@ -62,15 +89,26 @@ class SQLiteBackedProductMixin:
 
         metadata_bin = json.dumps(metadata).encode('utf-8')
         cur = self.client.connection.cursor()
+        schema = self._get_schema()
 
-        if self._get_schema():
+        if schema is not False:
+            if schema is None:
+                cur.execute('DELETE FROM _metadata WHERE name = ?',
+                            (self.name,))
+            else:
+                cur.execute('DELETE FROM _metadata '
+                            'WHERE name = ? AND schema = ?',
+                            (self.name, schema))
+
+            # we cannot rely on INSERT INTO since NULL schema values are
+            # allowed
             query = """
-                REPLACE INTO _metadata(metadata, name, schema)
+                INSERT INTO _metadata (metadata, name, schema)
                 VALUES(?, ?, ?)
             """
             cur.execute(query, (sqlite3.Binary(metadata_bin),
                                 self.name,
-                                self._get_schema()))
+                                schema))
         else:
             query = """
                 REPLACE INTO _metadata(metadata, name)
@@ -107,12 +145,21 @@ class SQLiteRelation(SQLiteBackedProductMixin, Product):
 
     Parameters
     ----------
-    identifier: tuple of length 3
+    identifier: tuple of length 3 or 2
         A tuple with (schema, name, kind) where kind must be either 'table'
-        or 'view'
+        or 'view'. If passed a tuple with length 2, schema is assumed None.
+        Schemas in SQLite represent other databases when using the ATTACH
+        command.
     client: ploomber.clients.DBAPIClient or SQLAlchemyClient, optional
         The client used to connect to the database. Only required
         if no dag-level client has been declared using dag.clients[class]
+
+
+    Examples
+    --------
+    >>> from ploomber.products import SQLiteRelation
+    >>> relation = SQLiteRelation(('schema', 'some_table', 'table'))
+    >>> str(relation) # returns qualified name
     """
 
     def __init__(self, identifier, client=None):
@@ -122,20 +169,6 @@ class SQLiteRelation(SQLiteBackedProductMixin, Product):
     def _init_identifier(self, identifier):
         return SQLRelationPlaceholder(identifier)
 
-    @property
-    def client(self):
-        # FIXME: this nested reference looks ugly
-        if self._client is None:
-            default = self.task.dag.clients.get(type(self))
-
-            if default is None:
-                raise ValueError('{} must be initialized with a client'
-                                 .format(type(self).__name__))
-            else:
-                self._client = default
-
-        return self._client
-
     def exists(self):
         query = """
         SELECT name
@@ -176,7 +209,7 @@ class SQLiteRelation(SQLiteBackedProductMixin, Product):
 
 # FIXME: self._identifier should not be accessed direclty since it might
 # be a placeholder
-class PostgresRelation(Product):
+class PostgresRelation(ProductWithClientMixin, Product):
     """A PostgreSQL relation
 
     Parameters
@@ -187,6 +220,12 @@ class PostgresRelation(Product):
     client: ploomber.clients.DBAPIClient or SQLAlchemyClient, optional
         The client used to connect to the database. Only required
         if no dag-level client has been declared using dag.clients[class]
+
+    Examples
+    --------
+    >>> from ploomber.products import PostgresRelation
+    >>> relation = PostgresRelation(('schema', 'some_table', 'table'))
+    >>> str(relation) # returns qualified name
     """
     # FIXME: identifier has schema as optional but that introduces ambiguity
     # when fetching metadata and checking if the table exists so maybe it
@@ -199,19 +238,6 @@ class PostgresRelation(Product):
     def _init_identifier(self, identifier):
         return SQLRelationPlaceholder(identifier)
 
-    @property
-    def client(self):
-        if self._client is None:
-            default = self.task.dag.clients.get(type(self))
-
-            if default is None:
-                raise ValueError('{} must be initialized with a client'
-                                 .format(type(self).__name__))
-            else:
-                self._client = default
-
-        return self._client
-
     def fetch_metadata(self):
         cur = self.client.connection.cursor()
 
