@@ -5,15 +5,25 @@ from collections import abc
 class Params(abc.Mapping):
     """
     Read-only mapping to represent params passed in Task constructor. It
-    initializes with a copy of the passed dictionary
+    initializes with a copy of the passed dictionary. Cannot be initialized
+    with a key "upstream" nor "product" as they are added upon Task rendering
     """
 
-    def __init__(self, dict_=None):
-        if dict_ is None:
+    def __init__(self, params=None):
+        if params is None:
             self._dict = {}
         else:
+            if 'upstream' in params:
+                raise ValueError('Task params cannot be initialized with an '
+                                 '"upstream" key as it automatically added '
+                                 'upon rendering')
 
-            self._dict = copy(dict_)
+            if 'product' in params:
+                raise ValueError('Task params cannot be initialized with an '
+                                 '"product" key as it automatically added '
+                                 'upon rendering')
+
+            self._dict = copy(params)
 
     def to_dict(self):
         return copy(self._dict)
