@@ -55,8 +55,8 @@ class SQLScript(Task):
     def run(self):
         return self.client.execute(str(self.source))
 
-    def _init_source(self, source):
-        return SQLScriptSource(source)
+    def _init_source(self, source, kwargs):
+        return SQLScriptSource(source, **kwargs)
 
 
 class SQLDump(Task):
@@ -114,8 +114,8 @@ class SQLDump(Task):
             raise ValueError('{} must be initialized with a client'
                              .format(type(self).__name__))
 
-    def _init_source(self, source):
-        return SQLQuerySource(source)
+    def _init_source(self, source, kwargs):
+        return SQLQuerySource(source, **kwargs)
 
     def run(self):
         source_code = str(self.source)
@@ -170,7 +170,7 @@ class SQLTransfer(Task):
     product: ploomber.products.Product
         Product generated upon successful execution. For SQLTransfer, usually
         product.client != task.client. task.client represents the data source
-        while product.client represents the data destination.
+        while product.client represents the data destination
     dag: ploomber.DAG
         A DAG to add this task to
     name: str
@@ -211,9 +211,9 @@ class SQLTransfer(Task):
 
         self.chunksize = chunksize
 
-    def _init_source(self, source):
+    def _init_source(self, source, kwargs):
         # TODO: this shoule be a FileSource
-        return SQLQuerySource(source)
+        return SQLQuerySource(source, **kwargs)
 
     def run(self):
         source_code = str(self.source)
@@ -234,39 +234,53 @@ class SQLTransfer(Task):
                       index=False)
 
 
-# FIXME: support other data formats apart from parquet
 class SQLUpload(Task):
     """
-    Upload data to a SQL database from a parquet file (Note: this relies on
-    pandas, only use it for small to medium size datasets)
+    Upload data to a SQL database from a parquet or a csv file. Note: this
+    task relies uses pandas.to_sql which introduces some overhead. Only use it
+    for small to medium size datasets. Each database usually come with a tool
+    to upload data efficiently. If you are using PostgreSQL, check out the
+    PostgresCopyFrom task.
 
     Parameters
     ----------
-    source: str or pathlib.Path
-        Path to parquet file to upload
-    product: ploomber.products.Product
-        Product generated upon successful execution. For SQLTransfer, usually
-        product.client != task.client. task.client represents the data source
-        while product.client represents the data destination.
-    dag: ploomber.DAG
+    source : str or pathlib.Path
+        Path to parquet or a csv file to upload
+
+    product : ploomber.products.Product
+        Product generated upon successful execution. The client for the product
+        must be in the target database, where as task.client should be a client
+        in the source database.
+
+    dag : ploomber.DAG
         A DAG to add this task to
-    name: str
+
+    name : str
         A str to indentify this task. Should not already exist in the dag
+
     client: ploomber.clients.SQLAlchemyClient, optional
         The client used to connect to the database. Only required
         if no dag-level client has been declared using dag.clients[class]
-    params: dict, optional
+
+    params : dict, optional
         Parameters to pass to the script, by default, the callable will
         be executed with a "product" (which will contain the product object).
         It will also include a "upstream" parameter if the task has upstream
         dependencies along with any parameters declared here. The source
         code is converted to a jinja2.Template for passing parameters,
         refer to jinja2 documentation for details
-    chunksize: int, optional
+
+    chunksize : int, optional
         Number of rows to transfer on every chunk
+
     io_handler : callable, optional
         A Python callable to read the source file,
-        if None, it will tried to be infered from the source file extension
+        if None, it will tried to be inferred from the source file extension
+
+    to_sql_kwargs : dict, optional
+        Keyword arguments passed to the pandas.DataFrame.to_sql function,
+        one useful parameter is "if_exists", which determines if the inserted
+        rows should replace the table or just be appended
 
     Notes
     -----
@@ -279,7 +293,8 @@ class SQLUpload(Task):
 
     @requires(['pandas'], 'SQLUpload')
     def __init__(self, source, product, dag, name=None, client=None,
-                 params=None, chunksize=None, io_handler=None):
+                 params=None, chunksize=None, io_handler=None,
+                 to_sql_kwargs=None):
         params = params or {}
         super().__init__(source, product, dag, name, params)
 
@@ -291,9 +306,10 @@ class SQLUpload(Task):
 
         self.chunksize = chunksize
         self.io_handler = io_handler
+        self.to_sql_kwargs = to_sql_kwargs or {}
 
-    def _init_source(self, source):
-        return FileSource(str(source))
+    def _init_source(self, source, kwargs):
+        return FileSource(str(source), **kwargs)
 
     def run(self):
         product = self.params['product']
@@ -319,10 +335,9 @@ class SQLUpload(Task):
         self._logger.info('Done reading data...')
 
         df.to_sql(name=product.name,
-                  con=product.client.engine,
+                  con=self.client.engine,
                   schema=product.schema,
-                  if_exists='replace',
-                  index=False)
+                  **self.to_sql_kwargs)
 
 
 # TODO: provide more flexibility to configure the COPY statement
@@ -361,8 +376,8 @@ class PostgresCopyFrom(Task):
         self.null = null
         self.columns = columns
 
-    def _init_source(self, source):
-        return FileSource(str(source))
+    def _init_source(self, source, kwargs):
+        return FileSource(str(source), **kwargs)
 
     def run(self):
         product = self.params['product']
@@ -371,7 +386,7 @@ class PostgresCopyFrom(Task):
         # create the table
         self._logger.info('Creating table...')
         df.head(0).to_sql(name=product.name,
-                          con=product.client.engine,
+                          con=self.client.engine,
                           schema=product.schema,
                           if_exists='replace',
                           index=False)
